package control

import (
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-cli/internal/key"
	commonCmd "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/internal/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control"
	"github.com/spf13/cobra"
)

const (
	dumpFilepathFlag     = "path"
	dumpIgnoreErrorsFlag = "no-errors"
)

var dumpShardCmd = &cobra.Command{
	Use:   "dump",
	Short: "Dump objects from shard",
	Long:  "Dump objects from shard to a file",
	Run:   dumpShard,
}

func dumpShard(cmd *cobra.Command, _ []string) {
	pk := key.Get(cmd)

	body := new(control.DumpShardRequest_Body)
	body.SetShardID(getShardID(cmd))

	p, _ := cmd.Flags().GetString(dumpFilepathFlag)
	body.SetFilepath(p)

	ignore, _ := cmd.Flags().GetBool(dumpIgnoreErrorsFlag)
	body.SetIgnoreErrors(ignore)

	req := new(control.DumpShardRequest)
	req.SetBody(body)

	signRequest(cmd, pk, req)

	cli := getClient(cmd, pk)

	var resp *control.DumpShardResponse
	var err error
	err = cli.ExecRaw(func(client *client.Client) error {
		resp, err = control.DumpShard(client, req)
		return err
	})
	commonCmd.ExitOnErr(cmd, "rpc error: %w", err)

	verifyResponse(cmd, resp.GetSignature(), resp.GetBody())

	cmd.Println("Shard has been dumped successfully.")
}

func initControlDumpShardCmd() {
	initControlFlags(dumpShardCmd)

	flags := dumpShardCmd.Flags()
	flags.String(shardIDFlag, "", "Shard ID in base58 encoding")
	flags.String(dumpFilepathFlag, "", "File to write objects to")
	flags.Bool(dumpIgnoreErrorsFlag, false, "Skip invalid/unreadable objects")

	_ = dumpShardCmd.MarkFlagRequired(shardIDFlag)
	_ = dumpShardCmd.MarkFlagRequired(dumpFilepathFlag)
	_ = dumpShardCmd.MarkFlagRequired(controlRPC)
}
