package intermediate

import (
	"context"
	"errors"
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/core/netmap"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/reputation"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/reputation/eigentrust"
	eigencalc "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/reputation/eigentrust/calculator"
	eigentrustctrl "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/reputation/eigentrust/controller"
	apireputation "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/reputation"
)

// InitialTrustSource is an implementation of the
// reputation/eigentrust/calculator's InitialTrustSource interface.
type InitialTrustSource struct {
	NetMap netmap.Source
}

var ErrEmptyNetMap = errors.New("empty NepMap")

// InitialTrust returns `initialTrust` as an initial trust value.
func (i InitialTrustSource) InitialTrust(apireputation.PeerID) (reputation.TrustValue, error) {
	nm, err := i.NetMap.GetNetMap(1)
	if err != nil {
		return reputation.TrustZero, fmt.Errorf("failed to get NetMap: %w", err)
	}

	nodeCount := reputation.TrustValueFromFloat64(float64(len(nm.Nodes())))
	if nodeCount == 0 {
		return reputation.TrustZero, ErrEmptyNetMap
	}

	return reputation.TrustOne.Div(nodeCount), nil
}

// DaughtersTrustCalculator wraps EigenTrust calculator and implements the
// eigentrust/calculator's DaughtersTrustCalculator interface.
type DaughtersTrustCalculator struct {
	Calculator *eigencalc.Calculator
}

// Calculate converts and passes values to the wrapped calculator.
func (c *DaughtersTrustCalculator) Calculate(ctx context.Context, iterCtx eigentrustctrl.IterationContext) {
	calcPrm := eigencalc.CalculatePrm{}
	epochIteration := eigentrust.EpochIteration{}

	epochIteration.SetEpoch(iterCtx.Epoch())
	epochIteration.SetI(iterCtx.I())

	calcPrm.SetLast(iterCtx.Last())
	calcPrm.SetEpochIteration(epochIteration)

	c.Calculator.Calculate(ctx, calcPrm)
}
