package allocator

import (
	"testing"

	"github.com/stretchr/testify/require"
)

func TestBestFit1(t *testing.T) {
	a := NewBestFit(4194304)
	testGet(t, a, 1, NewRegion(0, 1))
	testGet(t, a, 1, NewRegion(1, 1))
	testGet(t, a, 1, NewRegion(2, 1))
	testGet(t, a, 1, NewRegion(3, 1))
}

func TestBestFit(t *testing.T) {
	a := NewBestFit(4096)
	testGet(t, a, 1024, NewRegion(0, 1024))
	testGet(t, a, 1024, NewRegion(1024, 1024))

	require.NoError(t, a.Free(NewRegion(0, 1024)))
	testGet(t, a, 1024, NewRegion(0, 1024))

	require.NoError(t, a.Free(NewRegion(0, 1024)))
	testGet(t, a, 1025, NewRegion(2048, 1025))
	testGet(t, a, 512, NewRegion(2048+1025, 512))
	testGet(t, a, 512, NewRegion(0, 512))
	testGet(t, a, 512, NewRegion(512, 512))

	_, err := a.Get(512)
	require.True(t, err == ErrOOM)
	testGet(t, a, 511, NewRegion(2048+1025+512, 511))

	require.NoError(t, a.Free(NewRegion(0, 1024)))
	require.NoError(t, a.Free(NewRegion(2048, 2048)))
	require.NoError(t, a.Free(NewRegion(1024, 1024)))
	testGet(t, a, 4096, NewRegion(0, 4096))
}

func BenchmarkBestFit(b *testing.B) {
	benchmarkAllocator(b, NewBestFit)
}

func benchmarkAllocator[A Allocator](b *testing.B, constructor func(uint64) A) {
	b.Run("by one", func(b *testing.B) {
		a := constructor(uint64(b.N))
		for i := 0; i < b.N; i++ {
			_, err := a.Get(1)
			if err != nil {
				b.Fatal(err)
			}
		}
	})
}
