package allocator

import (
	"github.com/RoaringBitmap/roaring"
)

// Bitmap allocates chunks of a constant size.
type Bitmap struct {
	free      roaring.Bitmap
	allocated roaring.Bitmap
}

func NewBitmap(capacity uint64) *Bitmap {
	b := &Bitmap{}
	b.free.AddRange(0, capacity)
	return b
}

func (b *Bitmap) Get(size int) (Region, error) {
	if size != 1 {
		panic("not implemented")
	}
	r := b.free.Minimum()
	b.free.Remove(r)
	b.allocated.Add(r)
	return NewRegion(uint64(r), uint64(size)), nil
}

func (b *Bitmap) Free(region Region) error {
	b.allocated.RemoveRange(region.Offset(), region.End())
	b.free.AddRange(region.Offset(), region.End())
	return nil
}
