package goodstor

import (
	"os"
	"path/filepath"

	"codeberg.org/fyrchik/uring/loop"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/util"
)

const idToStringBase = 16

type nameIDPair struct {
	name string
	id   uint16
}

func (s *Storage) Open(readOnly bool) error {
	s.readOnly = readOnly

	if s.loopSize != 0 {
		lp, err := loop.New(s.loopSize, &s.loopParams)
		if err != nil {
			return err
		}
		s.loop = lp
	}

	if !readOnly {
		if err := util.MkdirAllX(s.path, os.ModePerm); err != nil {
			return err
		}
	}

	p := filepath.Join(s.path, "backend")
	b, err := s.openSlab(p)
	if err != nil {
		return err
	}
	if err := b.Fallocate(int64(s.capacity)); err != nil {
		return err
	}

	s.backend = b
	return nil
}

func (s *Storage) Init() error {
	return nil
}

func (s *Storage) Close() error {
	if s.backend != nil {
		if err := s.backend.Close(); err != nil {
			return err
		}
		s.backend = nil
	}

	if s.loop != nil {
		if err := s.loop.Close(); err != nil {
			return err
		}
		s.loop = nil
	}

	return nil
}
