package goodstor

import (
	"context"
	"errors"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
)

var errInvalidStorageID = errors.New("invalid storage ID")

func (s *Storage) Get(_ context.Context, prm common.GetPrm) (common.GetRes, error) {
	if prm.StorageID == nil {
		return common.GetRes{}, nil
	}

	offset, length, err := parseStorageID(prm.StorageID)
	if err != nil {
		return common.GetRes{}, err
	}

	data := make([]byte, length)
	err = s.backend.ReadAt(data, int64(offset*s.blockSize))
	if err != nil {
		return common.GetRes{}, err
	}

	obj := objectSDK.New()
	if err := obj.Unmarshal(data); err != nil {
		return common.GetRes{}, err
	}

	return common.GetRes{Object: obj, RawData: data}, nil
}
