package goodstor

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
)

func (s *Storage) Put(prm common.PutPrm) (common.PutRes, error) {
	if s.readOnly {
		return common.PutRes{}, common.ErrReadOnly
	}
	if prm.RawData == nil {
		panic("unexpected")
	}

	up2 := s.blockSize
	for ; up2 <= uint64(len(prm.RawData)); up2 *= 2 {
	}

	s.allocMtx.Lock()
	r, err := s.allocator.Get(up2 / s.blockSize)
	s.allocMtx.Unlock()
	if err != nil {
		return common.PutRes{}, err
	}

	err = s.backend.WriteAt(prm.RawData, int64(r.Offset()*s.blockSize))
	if err != nil {
		s.allocMtx.Lock()
		_ = s.allocator.Free(r)
		s.allocMtx.Unlock()
	}

	storageID := marshalStorageID(r.Offset(), uint64(len(prm.RawData)))
	return common.PutRes{StorageID: storageID[:]}, nil
}
