package goodstor

import (
	"encoding/binary"
	"fmt"
	"os"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/uringstor/slab"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/util/logicerr"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

var errNotImplemented = logicerr.Wrap(fmt.Errorf("not implemented: %w", apistatus.ObjectNotFound{}))

func parseStorageID(storageID []byte) (uint64, uint64, error) {
	if len(storageID) != 10 {
		return 0, 0, errInvalidStorageID
	}

	offset := binary.LittleEndian.Uint64(storageID[:8])
	length := binary.LittleEndian.Uint16(storageID[8:])
	return offset, uint64(length), nil
}

func marshalStorageID(offset, length uint64) [10]byte {
	var storageID [10]byte
	binary.LittleEndian.PutUint64(storageID[:], offset)
	binary.LittleEndian.PutUint16(storageID[8:], uint16(length))
	return storageID
}

func assert(ok bool, msg string, args ...any) {
	if !ok {
		panic(fmt.Sprintf(msg, args...))
	}
}

func (s *Storage) openSlab(p string) (slab.Backend, error) {
	var b slab.Backend
	if s.loop == nil {
		b = slab.NewFileBackend(p)
	} else {
		b = slab.NewUringBackend(p, s.loop)
	}

	var flags int
	if s.readOnly {
		flags |= os.O_RDONLY
	} else {
		flags |= os.O_RDWR | os.O_CREATE
	}

	if !s.noSync {
		flags |= os.O_SYNC
	}
	return b, b.Open(flags)
}
