package uringstor

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/util/logicerr"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

func (s *Storage) Delete(prm common.DeletePrm) (common.DeleteRes, error) {
	if s.readOnly {
		return common.DeleteRes{}, common.ErrReadOnly
	}

	if prm.StorageID == nil {
		return common.DeleteRes{}, errNotImplemented
	}

	slabID, offset, err := parseStorageID(prm.StorageID)
	if err != nil {
		return common.DeleteRes{}, err
	}

	s.slabMtx.RLock()
	ss, ok := s.slabMap[slabID]
	s.slabMtx.RUnlock()

	if !ok {
		return common.DeleteRes{}, logicerr.Wrap(apistatus.ObjectNotFound{})
	}

	if ok, err := ss.Exists(offset); err == nil && !ok {
		return common.DeleteRes{}, logicerr.Wrap(apistatus.ObjectNotFound{})
	}

	return common.DeleteRes{}, ss.Delete(offset)
}
