package uringstor

import (
	"os"
	"path/filepath"
	"strconv"
	"testing"

	"codeberg.org/fyrchik/uring/loop"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/internal/blobstortest"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/util/logger"
	"go.uber.org/zap/zaptest"
)

func TestGeneric(t *testing.T) {
	_ = os.RemoveAll(t.Name())
	defer func() { _ = os.RemoveAll(t.Name()) }()

	helper := func(t *testing.T, dir string) common.Storage {
		return New(
			WithLogger(&logger.Logger{Logger: zaptest.NewLogger(t)}),
			WithRootPath(dir),
			WithMaxObjectSize(8192),
			WithSlabCapacity(1024*1024),
			WithLoopParams(32, loop.Params{RingCount: 1}))
	}

	var n int
	newUringStor := func(t *testing.T) common.Storage {
		dir := filepath.Join(t.Name(), strconv.Itoa(n))
		return helper(t, dir)
	}

	min := uint64(1024)
	max := uint64(4096)
	blobstortest.TestAll(t, newUringStor, min, max)

	t.Run("info", func(t *testing.T) {
		dir := filepath.Join(t.Name(), "info")
		blobstortest.TestInfo(t, func(t *testing.T) common.Storage {
			return helper(t, dir)
		}, Type, dir)
	})
}

func TestControl(t *testing.T) {
	_ = os.RemoveAll(t.Name())
	defer func() { _ = os.RemoveAll(t.Name()) }()

	var n int
	newUringStor := func(t *testing.T) common.Storage {
		dir := filepath.Join(t.Name(), strconv.Itoa(n))
		return New(
			WithLogger(&logger.Logger{Logger: zaptest.NewLogger(t)}),
			WithRootPath(dir),
			WithMaxObjectSize(4096),
			WithSlabCapacity(1024*1024),
			WithLoopParams(32, loop.Params{RingCount: 1}))
	}

	blobstortest.TestControl(t, newUringStor, 1024, 2048)
}
