package uringstor

import (
	"context"
	"errors"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
)

var errInvalidStorageID = errors.New("invalid storage ID")

func (s *Storage) Get(_ context.Context, prm common.GetPrm) (common.GetRes, error) {
	if prm.StorageID == nil {
		return common.GetRes{}, errNotImplemented
	}

	slabID, offset, err := parseStorageID(prm.StorageID)
	if err != nil {
		return common.GetRes{}, err
	}

	s.slabMtx.RLock()
	ss, ok := s.slabMap[slabID]
	s.slabMtx.RUnlock()

	if !ok {
		return common.GetRes{}, apistatus.ObjectNotFound{}
	}

	data, err := ss.Get(offset)
	if err != nil {
		return common.GetRes{}, err
	}

	obj := objectSDK.New()
	if err := obj.Unmarshal(data); err != nil {
		return common.GetRes{}, err
	}

	return common.GetRes{Object: obj, RawData: data}, nil
}
