package uringstor

import (
	"encoding/binary"
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/uringstor/slab"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/util/logicerr"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

var errNotImplemented = logicerr.Wrap(fmt.Errorf("not implemented: %w", apistatus.ObjectNotFound{}))

func parseStorageID(storageID []byte) (uint16, uint64, error) {
	if len(storageID) != 10 {
		return 0, 0, errInvalidStorageID
	}

	slabID := binary.LittleEndian.Uint16(storageID)
	offset := binary.LittleEndian.Uint64(storageID[2:])
	return slabID, offset, nil
}

func marshalStorageID(slabID uint16, offset uint64) [10]byte {
	var storageID [10]byte
	binary.LittleEndian.PutUint16(storageID[:], slabID)
	binary.LittleEndian.PutUint64(storageID[2:], offset)
	return storageID
}

func assert(ok bool, msg string, args ...any) {
	if !ok {
		panic(fmt.Sprintf(msg, args...))
	}
}

func (s *Storage) openSlab(p string, chunkSize uint32, slabCapacity uint64) (*slab.Slab, error) {
	var b slab.Backend
	if s.loop == nil {
		b = slab.NewFileBackend(p)
	} else {
		b = slab.NewUringBackend(p, s.loop)
	}

	ss, err := slab.New(slab.Params{
		Backend:   b,
		ChunkSize: chunkSize,
		Capacity:  slabCapacity,
		ReadOnly:  s.readOnly,
		NoSync:    s.noSync,
	})
	if err != nil {
		return nil, err
	}

	return ss, nil
}
