package audit

import (
	"context"
	"testing"
	"time"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client"
	auditAPI "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/audit"
	cidtest "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id/test"
	"github.com/nspcc-dev/neo-go/pkg/crypto/keys"
	"github.com/nspcc-dev/neo-go/pkg/util"
	"github.com/stretchr/testify/require"
)

func TestAuditResults(t *testing.T) {
	t.Skip()
	const epoch = 11

	endpoint := "http://morph_chain.frostfs.devenv:30333"
	sAuditHash := "cdfb3dab86e6d60e8a143d9e2ecb0b188f3dc2eb"
	irKeyWIF := "L3o221BojgcCPYgdbXsm6jn7ayTZ72xwREvBHXKknR8VJ3G4WmjB"

	key, err := keys.NewPrivateKeyFromWIF(irKeyWIF)
	require.NoError(t, err)

	auditHash, err := util.Uint160DecodeStringLE(sAuditHash)
	require.NoError(t, err)

	morphClient, err := client.New(context.Background(), key, client.WithEndpoints(client.Endpoint{Address: endpoint}))
	require.NoError(t, err)

	auditClientWrapper, err := NewFromMorph(morphClient, auditHash, 0)
	require.NoError(t, err)

	id := cidtest.ID()

	var auditRes auditAPI.Result
	auditRes.ForEpoch(epoch)
	auditRes.SetAuditorKey(key.PublicKey().Bytes())
	auditRes.ForContainer(id)

	prm := PutPrm{}
	prm.SetResult(&auditRes)

	require.NoError(t, auditClientWrapper.PutAuditResult(prm))

	time.Sleep(5 * time.Second)

	list, err := auditClientWrapper.ListAuditResultIDByCID(epoch, id)
	require.NoError(t, err)
	require.Len(t, list, 1)

	savedAuditRes, err := auditClientWrapper.GetAuditResult(list[0])
	require.NoError(t, err)

	require.Equal(t, auditRes, savedAuditRes)
}
