package getsvc

import (
	"context"
	"errors"

	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/pkg/tracing"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
	"go.uber.org/zap"
)

func (exec *execCtx) executeLocal(ctx context.Context) {
	ctx, span := tracing.StartSpanFromContext(ctx, "getService.executeLocal")
	defer func() {
		span.End()
	}()

	var err error

	exec.collectedObject, err = exec.svc.localStorage.get(ctx, exec)

	var errSplitInfo *objectSDK.SplitInfoError
	var errRemoved apistatus.ObjectAlreadyRemoved
	var errOutOfRange apistatus.ObjectOutOfRange

	switch {
	default:
		exec.status = statusUndefined
		exec.err = err

		exec.log.Debug("local get failed",
			zap.String("error", err.Error()),
		)
	case err == nil:
		exec.status = statusOK
		exec.err = nil
		exec.writeCollectedObject(ctx)
	case errors.As(err, &errRemoved):
		exec.status = statusINHUMED
		exec.err = errRemoved
	case errors.As(err, &errSplitInfo):
		exec.status = statusVIRTUAL
		mergeSplitInfo(exec.splitInfo(), errSplitInfo.SplitInfo())
		exec.err = objectSDK.NewSplitInfoError(exec.infoSplit)
	case errors.As(err, &errOutOfRange):
		exec.status = statusOutOfRange
		exec.err = errOutOfRange
	}
}
